/* Copyright (C) 2009 Niels Serup
 * 
 * This file is part of Poddie.
 * 
 * Poddie is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * Poddie is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * A copy of the GNU General Public License can be found in the
 * file LICENSE.
 */

/*
 *                         componentsmenu.h  -  description
 *                         --------------------------------
 *   begin                : Mon Oct 12 2009
 *   copyright            : (C) 2009 by Niels Serup
 *   maintainer(s)        : Niels Serup (nqpz) <ns@metanohi.org>
 *   description          : Controls in-game components menu.
 *
 */

/// //////////////////////////////////////////////////////////////// ///
///                          PREPROCESSOR                            ///
/// //////////////////////////////////////////////////////////////// ///

#ifndef header_componentsmenu
#define header_componentsmenu

#include "precomp.h"

#if _MSC_VER > 1000
#pragma once
#endif

using namespace std;

/// //////////////////////////////////////////////////////////////// ///
///                    YET TO BE DEFINED CLASSES                     ///
/// //////////////////////////////////////////////////////////////// ///

class World;

/// //////////////////////////////////////////////////////////////// ///
///                           DEFINITION                             ///
/// //////////////////////////////////////////////////////////////// ///

struct ComponentEntry {
	string main;
	string type;
	string detailed;
	int depth;
	CL_Button *button;
	
	ComponentEntry(string main, string type, string detailed, int depth) : main(main), type(type), detailed(detailed), depth(depth) {}
};

class ComponentsMenu {
public:
	// Constructors
	ComponentsMenu() {};
	ComponentsMenu(World *world, CL_GUIManager *gui);
	
	// Destructor
	~ComponentsMenu();
	
	// Main variables
	World *world;
	CL_GUIManager *gui;
	//CL_ComponentManager *manager;
	CL_SlotContainer *slots;
	bool active, increase_visibility, full_visibility;
	float visibility;
	
	CL_Button *button_ok;
	
	vector<CL_Button *> buttons;
	
	//CL_Component *parent;
	
	
	///                 ///
	/// Important stuff ///
	///                 ///
	
	vector<ComponentEntry> comp_entries;
	
	
	///         ///
	/// Various ///
	///         ///
	
	bool ready;
	
	// Boxes
	CL_Rect menu_rect, window_rect;
	float menu_top, menu_left, menu_bottom, menu_right;
		
	// Functions
	void prepare();
	void update();
	
	void enter();
	void leave();
	void change_state();
	
	void click_component_button(int);
	
	void draw();
};

// Define it
#include "world.h"

#endif
