/* Copyright (C) 2009 Niels Serup
 * 
 * This file is part of Poddie.
 * 
 * Poddie is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * Poddie is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * A copy of the GNU General Public License can be found in the
 * file LICENSE.
 */

/*
 *                         gamemenu.h  -  description
 *                         --------------------------------
 *   begin                : Fri Sep 4 2009
 *   copyright            : (C) 2009 by Niels Serup
 *   maintainer(s)        : Niels Serup (nqpz) <ns@metanohi.org>
 *   description          : Controls in-game game menu.
 *
 */

/// //////////////////////////////////////////////////////////////// ///
///                          PREPROCESSOR                            ///
/// //////////////////////////////////////////////////////////////// ///

#ifndef header_gamemenu
#define header_gamemenu

#include "componentsmenu.h"
#include "precomp.h"

#if _MSC_VER > 1000
#pragma once
#endif

using namespace std;

/// //////////////////////////////////////////////////////////////// ///
///                    YET TO BE DEFINED CLASSES                     ///
/// //////////////////////////////////////////////////////////////// ///

class World;

/// //////////////////////////////////////////////////////////////// ///
///                           DEFINITION                             ///
/// //////////////////////////////////////////////////////////////// ///

struct ControlsEntry {
	string action;
	string technical;
	string key;
	
	ControlsEntry(string action, string technical, string key) : action(action), technical(technical), key(key) {}
};

class GameMenu {
public:
	// Constructors
	GameMenu() {};
	GameMenu(World *world, CL_GUIManager *gui, ComponentsMenu *comp_menu);
	
	// Destructor
	~GameMenu();
	
	// Main variables
	World *world;
	CL_GUIManager *gui;
	CL_ComponentManager *manager;
	CL_SlotContainer *slots;
	bool active, increase_visibility, full_visibility;
	float visibility;
	
	ComponentsMenu *comp_menu;
	
	
	///                                              ///
	/// Components and classes related to components ///
	///                                              ///
	
	// Tabs and subtabs
	vector<CL_Button *> tab;
	vector<CL_Component *> tab_content;
	vector<vector<CL_Button *> > subtab;
	vector<vector<CL_Component *> > subtab_content;
	
	// General components
	CL_Component *parent;
	CL_Component *bottom_components;
	CL_Label *restart_label;
	CL_Button *button_exit;
	
	// General/Main
	
	// General/Controls
	CL_TreeView *controls_tree;
	CL_Window *change_key;
	CL_InputBox *new_key;
	CL_Button *save_key;
		
	// General/Video
	CL_ListBox *resolution_box;
	CL_ListBox *quality_box;
	
	CL_Window *new_res;
	CL_InputBox *new_res_width;
	CL_InputBox *new_res_height;
	CL_Button *save_res_use;
	CL_Button *save_res;
	
	CL_RadioButton *fullscreen_toggle;
	CL_RadioButton *windowed_toggle;
	
	CL_InputBox *new_max_fps;
	CL_Button *save_max_fps;
	
	// General/Audio
	CL_Label *sfx_label;
	CL_Component *sfx_volume;
	CL_Button *sfx_adjust;
	
	///         ///
	/// Various ///
	///         ///
	
	// Background color
	unsigned char color_r, color_g, color_b;
	
	// Triangles
	double angle;
	float triangle_radius;
	
	// Tab information
	short current_tab;
	vector<short> current_subtab;
	vector<short> subtabs_amount;
	
	// Boxes
	CL_Rect menu_rect, window_rect;
	float menu_top, menu_left, menu_bottom, menu_right;
	
	// Controls
	vector<ControlsEntry> controls_list;
	CL_TreeItem *current_item;
	bool restart_bacause_of_key_change;
	
	// Video
	vector<string> available_resolutions;
	string new_graphics_quality;
	string temp_max_fps_str;
	
	// Volume adjusters
	int mouse_start_x;
	bool move_active;
	int sfx_len;
	int sfx_bars_used;
	vector<CL_Rect > sfx_boxes;
	
	// Functions
	void update();
	
	void load_controls(char[]);
	void click_controls_item(const CL_TreeNode&);
	
	void load_available_resolutions(char[]);
	
	void enter();
	void leave();
	void change_state();
	
	void click_tab(short);
	void click_subtab(short);
	
	void save_controls_key();
	
	void adjust_resolution(int);
	void save_new_resolution(bool);
	void adjust_quality(int);
	
	void click_max_fps_box();
	void check_click_max_fps_box();
	void save_use_max_fps();
	
	void update_sfx_label(float);
	
	void set_fullscreen(bool);
	void draw();
	
	void highlight_resolution_match();
	void highlight_quality_match();
	
	void start_move(const CL_InputEvent&);
	void end_move(const CL_InputEvent&);
	
	void save_settings();
};

// Define it
#include "world.h"

#endif
