/* Copyright (C) 2009 Sakse Dalum
 * 
 * This file is part of Poddie.
 * 
 * Poddie Server is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Poddie is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * A copy of the GNU General Public License can be found in the file
 * LICENSE. 
 */

/*
 *                         component.cpp  -  description
 *                         --------------------------------
 *   begin                : Sun Aug 16 2009
 *   copyright            : (C) 2009 by Sakse Dalum
 *   maintainer(s)        : Sakse "Don S" Dalum <don.s@yodanism.org>
 *   description          : Defines all components' functions, etc.
 *
 */

#include "component.h"

/***********************************************************************
*    *    *    *    *                              *    *    *    *    *
                            CON- & DESTRUCTORS
*    *    *    *    *                              *    *    *    *    *
***********************************************************************/

Component::Component(string type, string part, string number, int slot, Player &owner)
: type(type), part(part), number(number), slot(slot), owner(owner) {
	ostringstream ss;
	ss << "Poddie/" << type << "/" << part << "/" << number;
	sprite = new CL_Sprite(ss.str(), owner.world->resources);
	ss.str(""); // Set the contents of ss to "".
	if (part == "Engine") ss << "Poddie/" << type << "/" << part << "/Fire" << "/" << number;
	else ss << "Empty";
	
	activitySprite = new CL_Sprite(ss.str(), owner.world->resources);
	
	determine_values(type, part, number);
	active = false;
	activity = 0;
	sendStateIncrease = false;
	sendStateDecrease = false;
	sendStateActive = false;
	increase = false;
	decrease = false;
	
	if (part == "Engine") {
		sfx1 = CL_SoundBuffer("resources/sfx/engine1.wav");
		sfx1_session = sfx1.prepare();
		sfx1_session.set_speedfactor(0);
		sfx1_session.set_looping(true);		
		sfx1_session.play();
	}
}

Component::~Component() {
	sfx1_session.stop();
}

/***********************************************************************
*    *    *    *    *                              *    *    *    *    *
                            LOOPING FUNCTIONS
*    *    *    *    *                              *    *    *    *    *
***********************************************************************/

void Component::update() {
	
	if (part == "Engine") {
		// Play sound effects.
		sfx1_session.set_speedfactor(activity*2);
		sfx1_session.set_pan((owner.x - owner.world->player.at(owner.world->i_control)->x)/5000);
		if (sqrt(pow(owner.x - owner.world->player.at(owner.world->i_control)->x, 2)+pow(owner.y - owner.world->player.at(owner.world->i_control)->y, 2)) == 0) sfx1_session.set_volume(owner.world->sfx_volume*1);
		else if (sqrt(pow(owner.x - owner.world->player.at(owner.world->i_control)->x, 2)+pow(owner.y - owner.world->player.at(owner.world->i_control)->y, 2)) < 5000) sfx1_session.set_volume(owner.world->sfx_volume*(1-sqrt(pow(owner.x - owner.world->player.at(owner.world->i_control)->x, 2)+pow(owner.y - owner.world->player.at(owner.world->i_control)->y, 2))/5000));
		else sfx1_session.set_volume(0);
		if (activity < 0.01) sfx1_session.set_volume(0);
	}
	
	if (owner.does_control) {
		if (owner.world->keyboard.pressed(keyAction)) toggle();
		if (owner.world->keyboard.pressed(keyIncrease)) set_increase(true);
		if (owner.world->keyboard.pressed(keyDecrease)) set_decrease(true);
		if (owner.world->keyboard.released(keyIncrease)) set_increase(false);
		if (owner.world->keyboard.released(keyDecrease)) set_decrease(false);
	}
	
	if (!owner.alive) {
		// Stop playing the sound effects when the component is dead.
		if (part == "Engine") sfx1_session.stop();
	}
}

/***********************************************************************
*    *    *    *    *                              *    *    *    *    *
                               FUNCTIONS
*    *    *    *    *                              *    *    *    *    *
***********************************************************************/

////////////////////////////////////////////////////////////////////////
//                              ACTIVITY

//                               TOGGLE
void Component::toggle() {
	if (part == "Engine") {
		active = !active;
		if (active) owner.world->network_component_activate_send(owner.id, "active", true, slot);
		else owner.world->network_component_activate_send(owner.id, "active", false, slot);
	}
	if (part == "Weapon/Laser") {
		owner.world->network_component_activate_send(owner.id, "active", true, slot);
	}
}

//                              INCREASE
void Component::set_increase(bool val) {
	increase = val;
	if (increase) owner.world->network_component_activate_send(owner.id, "increase", true, slot);
	else owner.world->network_component_activate_send(owner.id, "increase", false, slot);
}

//                              DECREASE
void Component::set_decrease(bool val) {
	decrease = val;
	if (decrease) owner.world->network_component_activate_send(owner.id, "decrease", true, slot);
	else owner.world->network_component_activate_send(owner.id, "decrease", false, slot);
}

////////////////////////////////////////////////////////////////////////
//                                KEYS

//                             FROM STRING
void Component::set_keys(string actKey, string incKey, string decKey) {
	keyAction = owner.world->keyboard.string_to_int(actKey);
	keyIncrease = owner.world->keyboard.string_to_int(incKey);
	keyDecrease = owner.world->keyboard.string_to_int(decKey);
}

//                              FROM INT
void Component::set_keys(int actKey, int incKey, int decKey) {
	keyAction = actKey;
	keyIncrease = incKey;
	keyDecrease = decKey;
}

////////////////////////////////////////////////////////////////////////
//                               VALUES

void Component::determine_values(string type, string part, string number) {
	if (type == "Rusty") {
		if (part == "Base") {
			if (number == "Main1") {
				z = 55;
				set_keys(0, 0, 0);
			}
		}
		
		if (part == "Armour") {
			if (number == "Main1") {
				z = 20;
				set_keys(0, 0, 0);
			}
			if (number == "Front1") {
				z = 24;
				set_keys(0, 0, 0);
			}
			if (number == "Wings1") {
				z = 22;
				set_keys(0, 0, 0);
			}
			if (number == "WingsFront1") {
				z = 30;
				set_keys(0, 0, 0);
			}
		}
		
		if (part == "Engine") {
			if (number == "BackCenter1") {
				z = 31;
				set_keys(owner.world->config.read<string>("rusty_engine_backcenter1_toggle"), owner.world->config.read<string>("rusty_engine_backcenter1_increase"), owner.world->config.read<string>("rusty_engine_backcenter1_decrease"));
			}
			if (number == "BackRight1") {
				z = 15;
				set_keys(owner.world->config.read<string>("rusty_engine_backright1_toggle"), owner.world->config.read<string>("rusty_engine_backright1_increase"), owner.world->config.read<string>("rusty_engine_backright1_decrease"));
			}
			if (number == "BackLeft1") {
				z = 15;
				set_keys(owner.world->config.read<string>("rusty_engine_backleft1_toggle"), owner.world->config.read<string>("rusty_engine_backleft1_increase"), owner.world->config.read<string>("rusty_engine_backleft1_decrease"));
			}
			if (number == "MiddleRight1") {
				z = 40;
				set_keys(owner.world->config.read<string>("rusty_engine_middleright1_toggle"), owner.world->config.read<string>("rusty_engine_middleright1_increase"), owner.world->config.read<string>("rusty_engine_middleright1_decrease"));
			}
			if (number == "MiddleLeft1") {
				z = 40;
				set_keys(owner.world->config.read<string>("rusty_engine_middleleft1_toggle"), owner.world->config.read<string>("rusty_engine_middleleft1_increase"), owner.world->config.read<string>("rusty_engine_middleleft1_decrease"));
			}
		}
		
		if (part == "Weapon/Laser") {
			if (number == "MiddleRight1") {
				z = 50;
				set_keys(owner.world->config.read<string>("rusty_weapon_laser_middleright1_shoot"), owner.world->config.read<string>("rusty_weapon_laser_middleright1_increase"), owner.world->config.read<string>("rusty_weapon_laser_middleright1_decrease"));
			}
			if (number == "MiddleLeft1") {
				z = 50;
				set_keys(owner.world->config.read<string>("rusty_weapon_laser_middleleft1_shoot"), owner.world->config.read<string>("rusty_weapon_laser_middleleft1_increase"), owner.world->config.read<string>("rusty_weapon_laser_middleleft1_decrease"));
			}
		}
	}
}
