/* Copyright (C) 2009 Sakse Dalum
 * 
 * This file is part of Poddie.
 * 
 * Poddie Server is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Poddie is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * A copy of the GNU General Public License can be found in the file
 * LICENSE. 
 */

/*
 *                         component.cpp  -  description
 *                         --------------------------------
 *   begin                : Sun Aug 16 2009
 *   copyright            : (C) 2009 by Sakse Dalum
 *   maintainer(s)        : Sakse "Don S" Dalum <don.s@yodanism.org>
 *   description          : Defines all components' functions, etc.
 *
 */

/// //////////////////////////////////////////////////////////////// ///
///                          PREPROCESSOR                            ///
/// //////////////////////////////////////////////////////////////// ///

#ifndef header_poddiecomponent
#define header_poddiecomponent

#if _MSC_VER > 1000
#pragma once
#endif

#include "../precomp.h"
#include "../input/keyboard.h"
#include "player.h"
#include "../world.h"

/// //////////////////////////////////////////////////////////////// ///
///                    YET TO BE DEFINED CLASSES                     ///
/// //////////////////////////////////////////////////////////////// ///

class Player;
class World;

/// //////////////////////////////////////////////////////////////// ///
///                           DEFINITION                             ///
/// //////////////////////////////////////////////////////////////// ///

class Component {
public:
	// Constructor
	Component(std::string type, std::string part, std::string number, int slot, Player &owner);
	// Destructor
	~Component();
	
	// Object references
	Player &owner;
	
	// Component
	int slot;
	
	// Keys
	int keyAction, keyIncrease, keyDecrease;
	bool increase, decrease;
	void set_keys(std::string, std::string, std::string);
	void set_keys(int, int, int);
	void determine_values(std::string, std::string, std::string);
	
	// Activity
	bool active, sendStateIncrease, sendStateDecrease, sendStateActive;
	float activity;
	
	// Update
	void update();
	
	// Functions
	void toggle();
	void set_increase(bool);
	void set_decrease(bool);
	
	// Sprite
	std::string type, part, number;
	CL_Sprite *sprite;
	CL_Sprite *activitySprite;
	float z;
	
	// Sound
	CL_SoundBuffer sfx1;
	CL_SoundBuffer_Session sfx1_session;
};

#endif

