/* Copyright (C) 2009 Sakse Dalum
 * 
 * This file is part of Poddie.
 * 
 * Poddie Server is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Poddie is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * A copy of the GNU General Public License can be found in the file
 * LICENSE. 
 */

/*
 *                         missile.cpp  -  description
 *                         --------------------------------
 *   begin                : Sun Aug 16 2009
 *   copyright            : (C) 2009 by Sakse Dalum
 *   maintainer(s)        : Sakse "Don S" Dalum <don.s@yodanism.org>
 *   description          : Any flying missile.
 *
 */

#include "missile.h"

/***********************************************************************
*    *    *    *    *                              *    *    *    *    *
                            CON- & DESTRUCTORS
*    *    *    *    *                              *    *    *    *    *
***********************************************************************/

Missile::Missile(bool alive, float x, float y, float angle, float hspeed, float vspeed, Player *creator, World *world, CL_Sprite sprite) : GameObject(x, y, angle, hspeed, vspeed, world), sprite(sprite), alive(alive) {
	alive = true;
	collision = false;
	sprite_collision = new CL_Sprite("Poddie/Shield/Blue1", world->resources);
	radius = 0;
	collision_timer = CL_System::get_time() / 1000.0f;
}

Missile::Missile() {
}

Missile::~Missile() {
	
}

/***********************************************************************
*    *    *    *    *                              *    *    *    *    *
                           LOOPING FUNCTIONS
*    *    *    *    *                              *    *    *    *    *
***********************************************************************/

void Missile::update() {
	if (!collision) {
		x += hspeed;
		y += vspeed;
	}
	
	if (collision) {
		if (CL_System::get_time() / 1000.0f > collision_timer) {
			collision = false;
		}
	}
	
	if (x < 0 || y < 0 || x > 2048*3*15 || y > 2048*3*15) alive = false;
}

/***********************************************************************
*    *    *    *    *                              *    *    *    *    *
                              DRAW FUNCTION
*    *    *    *    *                              *    *    *    *    *
***********************************************************************/

void Missile::draw() {
	float draw_x = get_x()*world->zoom_factor-world->view_x;
	float draw_y = get_y()*world->zoom_factor-world->view_y;
	
	if (!collision) {
		sprite.set_scale(world->zoom_factor/world->quality, world->zoom_factor/world->quality);
		sprite.set_alignment(origin_center);
		sprite.set_angle(angle-90);
		sprite.draw(draw_x, draw_y, world->window->get_gc());
	}
	
	if (collision) {
		sprite_collision->set_alignment(origin_center);
		sprite_collision->set_alpha(((collision_timer - CL_System::get_time() / 1000.0f)/0.3f)/3.0f);
		float scale = (world->zoom_factor/world->quality)*1.0f/1024.0f*radius*2;
		sprite_collision->set_scale(scale, scale);
		sprite_collision->draw(draw_x, draw_y, world->window->get_gc());
	}
}
