/* Copyright (C) 2009 Sakse Dalum
 * 
 * This file is part of Poddie.
 * 
 * Poddie Server is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Poddie is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * A copy of the GNU General Public License can be found in the file
 * LICENSE. 
 */

/*
 *                         player.h  -  description
 *                         --------------------------------
 *   begin                : Tue Aug 13 2009
 *   copyright            : (C) 2009 by Sakse Dalum
 *   maintainer(s)        : Sakse "Don S" Dalum <don.s@yodanism.org>
 *   description          : Player object to control player variables,
 *                          components, etc.
 *
 */

/// //////////////////////////////////////////////////////////////// ///
///                          PREPROCESSOR                            ///
/// //////////////////////////////////////////////////////////////// ///

#ifndef header_player
#define header_player

#if _MSC_VER > 1000
#pragma once
#endif

#include "../precomp.h"
#include "../input/keyboard.h"
#include "component.h"
#include "gameobject.h"
#include "missile.h"
#include "../world.h"

/// //////////////////////////////////////////////////////////////// ///
///                    YET TO BE DEFINED CLASSES                     ///
/// //////////////////////////////////////////////////////////////// ///

class Component;

/// //////////////////////////////////////////////////////////////// ///
///                           DEFINITION                             ///
/// //////////////////////////////////////////////////////////////// ///

class Player : public GameObject {
public:
	// Constructors
	Player(float x, float y, float angle, World *world, int id);
	Player();
	
	// Destructor
	~Player();

	// Controller
	bool does_control;
	int id;
	
	// Status
	bool alive;
	float health;

	// Movement
	float prev_angle, angle, speed, acceleration;
	bool init;

	// Update function
	void update();

	// Components
	list <Component*> components;
	std::list<Component*>::iterator component[16];
	bool slot[16];
	
	void add_component(list<Component*>::iterator, Component*);
	void remove_component(list<Component*>::iterator);
	
	// Movement, position & direction
	float radius;
	
	void set_angle(float);
	void set_hspeed(float);
	void set_vspeed(float);
	void set_x(float);
	void set_y(float);
	
	float get_speed();

	// Simple Physics
	void add_vector(float, float);
	void apply_friction(float);
	
	// Sprite functions
	bool collision;
	CL_Sprite *sprite_collision;
	float collision_timer;
	
	void draw();
	
	// Text strings
	string get_speed_string();
	string get_position_string();
};

#endif
