/* Copyright (C) 2009 Sakse Dalum
 * 
 * This file is part of Poddie.
 * 
 * Poddie Server is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Poddie is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * A copy of the GNU General Public License can be found in the file
 * LICENSE. 
 */

/*
 *                         keyboard.cpp  -  description
 *                         --------------------------------
 *   begin                : Tue Aug 13 2009
 *   copyright            : (C) 2009 by Sakse Dalum
 *   maintainer(s)        : Sakse "Don S" Dalum <don.s@yodanism.org>
 *   description          : This object handles keyboard events.
 *
 */

#include "keyboard.h"

/***********************************************************************
*    *    *    *    *                              *    *    *    *    *
                            CON- & DESTRUCTORS
*    *    *    *    *                              *    *    *    *    *
***********************************************************************/

// Constructor
Keyboard::Keyboard() {
}

// Destructor
Keyboard::~Keyboard() {
	
}

/***********************************************************************
*    *    *    *    *                              *    *    *    *    *
                               FUNCTIONS
*    *    *    *    *                              *    *    *    *    *
***********************************************************************/

////////////////////////////////////////////////////////////////////////
//                          CHANGE KEY STATE

//----------------------------------------------------------------------
//!                            RESET KEYS
//----------------------------------------------------------------------

/* Function to reset a key. This function examines the key's state, and
 * change it to the next state in the sequence: If it's PRESSED, change
 * it to IS_PRESSED; if it is RELEASED, change it to NOT_PRESSED. */
void Keyboard::reset(int key_check) {	
	// Make sure the key state is not undefined.
	if (key[key_check] < 0 || key[key_check] > 3) key[key_check] = NOT_PRESSED;
	
	if (key[key_check] == PRESSED) key[key_check] = IS_PRESSED;
	if (key[key_check] == RELEASED) key[key_check] = NOT_PRESSED;
}

//----------------------------------------------------------------------
//!                             SET KEYS
//----------------------------------------------------------------------

//                              PRESSED
void Keyboard::set_pressed(int key_check) {	
	if (key[key_check] == NOT_PRESSED) key[key_check] = PRESSED;
}

//                              RELEASED
void Keyboard::set_released(int key_check) {
	if (key[key_check] >= NOT_PRESSED || key[key_check] <= RELEASED)
		key[key_check] = RELEASED;
}

////////////////////////////////////////////////////////////////////////
//                           GET KEY STATES

//                              PRESSED
bool Keyboard::pressed(int key_check) {
	if (key[key_check] == PRESSED) return true;
	else return false;
}

//                             IS PRESSED
bool Keyboard::is_pressed(int key_check) {
	if (CL_Keyboard::get_keycode(key_check)) return true;
	else return false;
}

//                              RELEASED
bool Keyboard::released(int key_check) {
	if (key[key_check] == RELEASED) return true;
	else return false;
}

//----------------------------------------------------------------------
//!                          TRANSLATE INPUT
//----------------------------------------------------------------------

//                           STRING TO INT
int Keyboard::string_to_int(std::string input) {
	int output = 0;
	
	if (input == "0") output = CL_KEY_0;
	if (input == "1") output = CL_KEY_1;
	if (input == "2") output = CL_KEY_2;
	if (input == "3") output = CL_KEY_3;
	if (input == "4") output = CL_KEY_4;
	if (input == "5") output = CL_KEY_5;
	if (input == "6") output = CL_KEY_6;
	if (input == "7") output = CL_KEY_7;
	if (input == "8") output = CL_KEY_8;
	if (input == "9") output = CL_KEY_9;
	
	if (input == "a") output = CL_KEY_A;
	if (input == "b") output = CL_KEY_B;
	if (input == "c") output = CL_KEY_C;
	if (input == "d") output = CL_KEY_D;
	if (input == "e") output = CL_KEY_E;
	if (input == "f") output = CL_KEY_F;
	if (input == "g") output = CL_KEY_G;
	if (input == "h") output = CL_KEY_H;
	if (input == "i") output = CL_KEY_I;
	if (input == "j") output = CL_KEY_J;
	if (input == "k") output = CL_KEY_K;
	if (input == "l") output = CL_KEY_L;
	if (input == "m") output = CL_KEY_M;
	if (input == "n") output = CL_KEY_N;
	if (input == "o") output = CL_KEY_O;
	if (input == "p") output = CL_KEY_P;
	if (input == "q") output = CL_KEY_Q;
	if (input == "r") output = CL_KEY_R;
	if (input == "s") output = CL_KEY_S;
	if (input == "t") output = CL_KEY_T;
	if (input == "u") output = CL_KEY_U;
	if (input == "v") output = CL_KEY_V;
	if (input == "w") output = CL_KEY_W;
	if (input == "x") output = CL_KEY_X;
	if (input == "y") output = CL_KEY_Y;
	if (input == "z") output = CL_KEY_Z;
	
	if (input == "space") output = CL_KEY_SPACE;
	if (input == "enter") output = CL_KEY_ENTER;
	if (input == "rctrl") output = CL_KEY_RCONTROL;
	if (input == "lctrl") output = CL_KEY_LCONTROL;
	if (input == "rshift") output = CL_KEY_RSHIFT;
	if (input == "lshift") output = CL_KEY_LSHIFT;
	if (input == "up") output = CL_KEY_UP;
	if (input == "down") output = CL_KEY_DOWN;
	if (input == "left") output = CL_KEY_LEFT;
	if (input == "right") output = CL_KEY_RIGHT;
	
	return output;
}

//----------------------------------------------------------------------
//!                          UPDATE KEYS
//----------------------------------------------------------------------

/* Calls ::reset() for all keys that we use. */
void Keyboard::update() {
	reset(0); //We reset 0 to make sure that non-keys do not accidentally activate.
		
	reset(CL_KEY_ESCAPE);	reset(CL_KEY_UP);		reset(CL_KEY_DOWN);
	reset(CL_KEY_LEFT);	reset(CL_KEY_RIGHT);	reset(CL_KEY_SPACE);
	
	reset(CL_KEY_0);		reset(CL_KEY_1);		reset(CL_KEY_2);
	reset(CL_KEY_3);		reset(CL_KEY_4);		reset(CL_KEY_5);
	reset(CL_KEY_6);		reset(CL_KEY_7);		reset(CL_KEY_8);
	reset(CL_KEY_9);
	
	reset(CL_KEY_A);		reset(CL_KEY_B);		reset(CL_KEY_C);
	reset(CL_KEY_D);		reset(CL_KEY_E);		reset(CL_KEY_F);
	reset(CL_KEY_G);		reset(CL_KEY_H);		reset(CL_KEY_I);
	reset(CL_KEY_J);		reset(CL_KEY_K);		reset(CL_KEY_L);
	reset(CL_KEY_M);		reset(CL_KEY_N);		reset(CL_KEY_O);
	reset(CL_KEY_P);		reset(CL_KEY_Q);		reset(CL_KEY_R);
	reset(CL_KEY_S);		reset(CL_KEY_T);		reset(CL_KEY_U);
	reset(CL_KEY_V);		reset(CL_KEY_W);		reset(CL_KEY_X);
	reset(CL_KEY_Y);		reset(CL_KEY_Z);
	
	reset(CL_KEY_NUMPAD_PLUS);			reset(CL_KEY_NUMPAD_MINUS);
}
