/* Copyright (C) 2009 Sakse Dalum
 * 
 * This file is part of Poddie.
 * 
 * Poddie Server is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Poddie is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * A copy of the GNU General Public License can be found in the file
 * LICENSE. 
 */

/*
 *                         world.h  -  description
 *                         --------------------------------
 *   begin                : Tue Aug 13 2009
 *   copyright            : (C) 2009 by Sakse Dalum
 *   maintainer(s)        : Sakse "Don S" Dalum <don.s@yodanism.org>
 *   description          : Controls all major functions of the game,
 *                          and takes care of updating objects etc.
 *
 */

/// //////////////////////////////////////////////////////////////// ///
///                          PREPROCESSOR                            ///
/// //////////////////////////////////////////////////////////////// ///
	
#ifndef header_world
#define header_world

#if _MSC_VER > 1000
#pragma once
#endif

#include "precomp.h"
#include "input/keyboard.h"
#include "input/keyboard.h"
#include "gamemenu.h"
#include "componentsmenu.h"
#include "gameobjects/component.h"
#include "gameobjects/player.h"
#include "gameobjects/missile.h"

/// //////////////////////////////////////////////////////////////// ///
///                    YET TO BE DEFINED CLASSES                     ///
/// //////////////////////////////////////////////////////////////// ///

class Player;
class Missile;

/// //////////////////////////////////////////////////////////////// ///
///                           DEFINITION                             ///
/// //////////////////////////////////////////////////////////////// ///

class World : public CL_Runnable {
public:
	// Constructor
	World(CL_ResourceManager *resources, CL_DisplayWindow *window, ConfigFile config, bool debug);
	
	// Destructor
	~World();
	
	// Resources, game window, configuration, etc.
	CL_ResourceManager *resources;
	CL_DisplayWindow *window;
	ConfigFile config;

	// Operations
	void run();
	bool quit;
	void on_quit() { quit = true; }
	bool debug;
	
	//!                         IN-GAME
	
	// In-game view & graphics quality
	float view_x, view_y;
	float zoom_factor;
	float quality;
	
	// In-game Objects
	vector < Player* > player;
	vector < Missile* > missile;

	// Rooms
	string room;
	
	// Surface
	int surface_hrepeat, surface_vrepeat;
	float surface_scale;
	
	// Player control
	bool has_control;
	int i_control; // The ID of the player we control
	
	//!                           GUI
	
	// Minimap
	float minimap_range, minimap_width, minimap_height, minimap_top;
	void update_minimap();
	void draw_minimap(CL_Surface&);
	
	// Activity states
	void draw_activity();
	
	// FPS
	float FPS, FPS_count, FPS_last, FPS_cur, frames;
	string FPS_string; /// TBR
	
	//!                   MUSIC, SOUND, ETC.
	
	// Sound effects
	float sfx_volume;
	
	// Music
	
	//!                      NETWORK
	vector < string > server_ip;
	vector < string > server_port;
	vector < string > server_name;
	
	float server_delay, last_call_delay, request_next_delay;
	string server_delay_string; /// TBR

	bool do_connect, can_connect, has_connected;
	
	CL_NetSession *netsession; //The session we use when communicating.
	CL_NetComputer server; //The server we have established a connection to.
	
	// Slots
	CL_SlotContainer slots;
	
	//Packets	
	CL_NetPacket *packet_component_activate;
	CL_NetPacket *packet_component_add;
	CL_NetPacket *packet_component_remove;
	
	CL_NetPacket *packet_delay_return;
	
	//Sending packets
	void network_component_activate_send(int, string, bool, int);
	void network_component_add_send(int, string, string, string, int);
	void network_component_remove_send(int, int);
	
	void network_delay_return_send(int);
	
	//Functions to call upon receiving packets
	void network_object_status_receive(CL_NetPacket &packet, CL_NetComputer &computer);
	void network_missile_create_receive(CL_NetPacket &packet, CL_NetComputer &computer);
	void network_collision_receive(CL_NetPacket &packet, CL_NetComputer &computer);
	void network_component_add_receive(CL_NetPacket &packet, CL_NetComputer &computer);
	void network_component_remove_receive(CL_NetPacket &packet, CL_NetComputer &computer);
	void network_component_activate_receive(CL_NetPacket &packet, CL_NetComputer &computer);
	void network_component_activity_receive(CL_NetPacket &packet, CL_NetComputer &computer);
	void network_delay_receive(CL_NetPacket &packet, CL_NetComputer &computer);
	void network_alive_receive(CL_NetPacket &packet, CL_NetComputer &computer);
	void network_dead_receive(CL_NetPacket &packet, CL_NetComputer &computer);

	//!                      Resources
	
	//Fonts
	CL_Font monospace;
	
	//Music
	CL_SoundBuffer ambient1;

	//!                        Input
	
	Keyboard keyboard;
	
	void keyboard_pressed(const CL_InputEvent &key);
	void keyboard_released(const CL_InputEvent &key);
};

#endif
